@@ -32,13 +32,11 @@
 // This is a copyrighted commercial software.
 // Please see COPYRIGHT file for details.
 
-/** \file
- */
-
 #include "AspellLibrary.h"
 #include "SpellChecker.h"
 
 #include "utils/SernaMessages.h"
+#include "utils/MsgBoxStream.h"
 #include "utils/utils_defs.h"
 #include "utils/Config.h"
 
@@ -85,21 +83,13 @@ static const ustring& Null(String::null());
 AspellLibrary::AspellLibrary() {}
 AspellLibrary::~AspellLibrary() {}
 
-namespace cfg {
+static const char ASPELL_CFG_VAR[]      = "aspell";
 
-static const char SPELL_DICTDIR_VAR[]  = "data/dict-dir";
-static const char SPELL_DATADIR_VAR[]  = "data/data-dir";
+static const char ASPELL_DICTDIR_VAR[]  = "dict-dir";
+static const char ASPELL_DATADIR_VAR[]  = "data-dir";
+static const char ASPELL_DLLNAME_VAR[]  = "lib";
 
-static const char SPELL_PWSDIR_VAR[]   = "pws-dir";
-static const char SPELL_DICT_VAR[]     = "default-dict";
-static const char SPELL_DLLNAME_VAR[]  = "data/lib";
-
-static const char SPELL_DICTDIR[]      = "dict";
-static const char SPELL_DATADIR[]      = "data";
-static const char SPELL_DICT[]         = "en";
-static const char SPELL_PWSDIR[]       = "";
-
-static const char SPELL_DICT_FLAG[]    = ".aspell_dictdir";
+static const char ASPELL_DICT_FLAG[]    = ".aspell_dictdir";
 
 #if defined(_WIN32)
 # if defined(_DEBUG) && !defined(NDEBUG)
@@ -116,12 +106,9 @@ static const char SPELL_DICT_FLAG[]    = ".aspell_dictdir";
 # define DLL_SFX ".so"
 #endif
 
-}
-
-static String get_cfg_value(const char* var, const PropertyNode* cfgNode = 0)
+static String get_cfg_value(const char* var)
 {
-    const PropertyNode* spPtn = cfgNode ? cfgNode :
-                                          config().getProperty("speller");
+    const PropertyNode* spPtn = config().getProperty("speller");
     if (spPtn) {
         if (const PropertyNode* varPtn = spPtn->getProperty(var))
             return varPtn->getString();
@@ -129,17 +116,6 @@ static String get_cfg_value(const char* var, const PropertyNode* cfgNode = 0)
     return String::null();
 }
 
-static const ustring& get_error(const ustring& s)
-{
-    return (0 < s.size()) ? s : Null;
-}
-
-static inline void reset_error(ustring& s)
-{
-    if (0 < s.size())
-        s.resize(0);
-}
-
 struct DictInfo {
     DictInfo(const char* c, const char* j) : code_(c), jargon_(j) {}
     nstring code_;
@@ -206,62 +182,25 @@ class AspellInstance : public AspellLibrary {
 public:
     AspellInstance();
     ~AspellInstance() { unload(); }
-    virtual bool loadLibrary(const COMMON_NS::ustring& libPath)
-    {
-        if (!isLoaded() || libPath != lib_path_) {
-            if (isLoaded())
-                unload();
-            if (!libPath.empty())
-                DynamicLibrary::load(libPath);
-        }
-        return isLoaded();
-    }
-    virtual void* resolveSym(const char* name) const
-    {
-        if (void* sym = resolve(name)) {
-            reset_error(sym_error_);
-            return sym;
-        }
-        sym_error_ = errorMsg();
-        return 0;
-    }
-    virtual const ustring& getLibError() const throw()
-    {
-        return get_error(lib_error_);
-    }
-    virtual const ustring& getSymError() const throw()
-    {
-        return get_error(sym_error_);
-    }
     virtual AspellConfig* getDefaultConfig() { return getConfig(); }
     virtual AspellSpeller* makeSpeller(const nstring& dict);
-    virtual bool getDictList(SpellChecker::Strings& si,
-                             SpellChecker::Status* ps = 0);
-    virtual void setDict(const nstring& dict)
-    {
-        dict_ = dict;
-        FUN(aspell_config_replace)(getConfig(), "lang", dict_.c_str());
-    }
+    virtual bool getDictList(SpellChecker::Strings& si) const;
     virtual const nstring& getDict() const { return dict_; }
     virtual const nstring& getEncoding(const nstring& dict);
     virtual const nstring& findDict(const nstring& dict);
-    virtual bool  setConfig(const PropertyNode* configNode);
+    virtual bool  setConfig();
     //!
 private:
     void            initConfig();
     //!
     AspellConfig*   getConfig();
     //!
+    bool            initialized_;
     nstring         dict_;
     DictInfoMap     dict_map_;
-    ustring         lib_error_;
     AspellConfig*   config_;
-    mutable ustring sym_error_;
     String          data_dir_;
     String          dict_dir_;
-    ustring         lib_path_;
-
-    const PropertyNode* cfgNode_;
 };
 
 // END_IGNORE_LITERALS
@@ -284,21 +223,6 @@ static String& strip_path_chars(String& path)
     return path;
 }
 
-static bool get_aspell_dir(const PropertyNode* cfgNode, const ustring& base,
-                           const char* var, const char* val, String& aspellDir)
-{
-    aspellDir = cfgNode->getSafeProperty(var)->getString();
-    PathName dir(aspellDir);
-    if (dir.exists())
-        return true;
-    dir.assign(base).append(val);
-    if (dir.exists()) {
-        aspellDir = dir.name();
-        return true;
-    }
-    return false;
-}
-
 static String aspell_dir_error(const String& pfx, const String& dictDir)
 {
     return pfx + from_latin1(NOTR(" '")) + dictDir + from_latin1(NOTR("' ")) +
@@ -323,27 +247,26 @@ static String find_system_aspell_lib()
     return String();
 }
 
-bool AspellInstance::setConfig(const PropertyNode* cfgNode)
+bool AspellInstance::setConfig()
 {
     DBG_TRACE(DBG_DEFAULT_TAG) << NOTR("AspellInstance::setConfig(), this: ")
                                << this << std::endl;
-    if (cfgNode_)
+    if (initialized_)
         return true;
-
-    cfgNode_ = cfgNode;
-    using namespace cfg;
-    if (!cfgNode_) {
-        lib_error_ = String(tr("Invalid configuration data"));
+    const PropertyNode* cfgNode = 
+        config().root()->getProperty(SPELL_CFG_VAR, ASPELL_CFG_VAR); 
+    if (!cfgNode) {
+        setLibError(tr("Missing aspell configuration data"));
         return false;
     }
-
+    initialized_ = true;
     bool isSysAspell = false;
-    String path(cfgNode_->getSafeProperty(SPELL_DLLNAME_VAR)->getString());
+    String path(cfgNode->getString(ASPELL_DLLNAME_VAR));
     PathName libPath(path);
-    if (!PathName::exists(path)) {
-        path = cfgNode_->getSafeProperty(NOTR("resolved-path"))->getString();
+    if (path.isEmpty() || !PathName::exists(path)) {
+        path = cfgNode->parent()->getString("#resolved-path");
         PathName aspellDll(path);
-        aspellDll.append( DLL_BASE DLL_SFX );
+        aspellDll.append(DLL_BASE DLL_SFX);
         if (!aspellDll.exists()) {
             path = find_system_aspell_lib();
             isSysAspell = true;
@@ -355,33 +278,32 @@ bool AspellInstance::setConfig(const PropertyNode* cfgNode)
          << '\'' << std::endl;
 
     if (path.empty()) {
-        lib_error_ = String(tr("Cannot find aspell shared library"));
+        setLibError(tr("Cannot find aspell shared library"));
         return false;
     }
     else if (!loadLibrary(path)) {
-        lib_error_ = String(tr("Cannot load"));
-        str_append(lib_error_, NOTR(" '")).append(fileName());
-        str_append(lib_error_, NOTR("': ")).append(errorMsg());
-        ustring::iterator it(remove_if(lib_error_.begin(),
-                                       lib_error_.end(),
+        String lib_error(String(tr("Cannot load")) + "'" + fileName() +
+            "': " + errorMsg());
+        ustring::iterator it(remove_if(lib_error.begin(),
+                                       lib_error.end(),
                                        bind2nd(less<Char>(), ' ')));
-        lib_error_.erase(it, lib_error_.end());
+        lib_error.erase(it, lib_error.end());
+        setLibError(lib_error);
         return false;
     }
     path = PathName(fileName()).dirname().name();
-    reset_error(lib_error_);
+    setLibError();
 
     if (!isSysAspell) {
-        String dictDir;
-        if (!get_aspell_dir(cfgNode_, path, SPELL_DICTDIR_VAR, SPELL_DICTDIR,
-                            dictDir)) {
-            lib_error_ = aspell_dir_error(String(tr("Dictionary directory")),
-                                          dictDir);
+        String dictDir = cfgNode->getString(ASPELL_DICTDIR_VAR);
+        if (!PathName::exists(dictDir)) {
+            setLibError(aspell_dir_error(String(tr("Dictionary directory")),
+                                          dictDir));
             return false;
         }
 #if defined(__APPLE__)
         PathName dictFlag(dictDir);
-        dictFlag.append(cfg::SPELL_DICT_FLAG);
+        dictFlag.append(ASPELL_DICT_FLAG);
         DDBG << "DictFlag: " << dictFlag.name() << std::endl;
         if (!dictFlag.exists()) {
             int byteorder = 0;
@@ -397,22 +319,21 @@ bool AspellInstance::setConfig(const PropertyNode* cfgNode)
             }
             DDBG << "New dictDir: " << dictDir << std::endl;
             dictFlag = dictDir;
-            dictFlag.append(cfg::SPELL_DICT_FLAG);
+            dictFlag.append(ASPELL_DICT_FLAG);
             DDBG << "New DictFlag: " << dictFlag.name() << std::endl;
         }
         if (!dictFlag.exists()) {
-            lib_error_ = aspell_dir_error(String(tr("Dictionary directory")),
-                                          dictDir);
+            setLibError(aspell_dir_error(String(tr("Dictionary directory")),
+                                          dictDir));
             return false;
         }
 #endif
-
         dict_dir_ = strip_path_chars(dictDir);
 
-        String dataDir;
-        if (!get_aspell_dir(cfgNode_, path, SPELL_DATADIR_VAR, SPELL_DATADIR,
-                            dataDir)) {
-            lib_error_ = aspell_dir_error(String(tr("Data directory")), dataDir);
+        String dataDir = cfgNode->getString(ASPELL_DATADIR_VAR);
+        if (!PathName::exists(dataDir)) {
+            setLibError(aspell_dir_error(String(tr("Data directory")), 
+                dataDir));
             return false;
         }
         data_dir_ = strip_path_chars(dataDir);
@@ -428,9 +349,7 @@ AspellConfig* AspellInstance::getConfig()
 
 static inline void report_error(SernaMessages::Messages m)
 {
-    SpellErrorInfoPtr ip(new SpellChecker::Error::Info);
-    *ip << m << Message::L_ERROR;
-    throw AspellErr(ip.get());
+    msgbox_stream() << m << Message::L_ERROR;
 }
 
 #if defined(__APPLE__)
@@ -487,7 +406,7 @@ void AspellInstance::initConfig()
         }
     }
 
-    nstring tmp(latin1(get_cfg_value(cfg::SPELL_DICT_VAR)));
+    nstring tmp(latin1(get_cfg_value(SPELL_DICT_VAR)));
     if (dict_.empty() || dict_ != tmp) {
         if (0 != dict_map_.count(tmp))
             dict_ = tmp;
@@ -499,7 +418,7 @@ void AspellInstance::initConfig()
 }
 
 AspellInstance::AspellInstance()
- :  config_(0), cfgNode_(0)
+ :  initialized_(false), config_(0)
 {
 }
 
@@ -529,37 +448,28 @@ const nstring& AspellInstance::findDict(const nstring& dict)
     return dict_;
 }
 
-bool AspellInstance::getDictList(SpellChecker::Strings& strings,
-                                 SpellChecker::Status* ps)
+bool AspellInstance::getDictList(SpellChecker::Strings& strings) const
 {
-    initConfig();
-
+    const_cast<AspellInstance*>(this)->initConfig();
     if (!dict_map_.empty()) {
         DictInfoMap::const_iterator it = dict_map_.begin();
         for (; it != dict_map_.end(); ++it)
             strings.push_back(from_latin1(it->first.c_str(), it->first.size()));
         return true;
     }
-    if (0 != ps) {
-        SpellErrorInfoPtr ip(new SpellChecker::Error::Info);
-        *ip << SernaMessages::spellCheckerNoDictionaries << Message::L_ERROR;
-        ps->reset(ip.get());
-    }
     return false;
 }
 
 static inline void report_error(AspellCanHaveError* pche)
 {
-    const char* msg = FUN(aspell_error_message)(pche);
-    AspellErr err(msg);
-    throw err;
+    String msg(FUN(aspell_error_message)(pche));
+    msgbox_stream() << SernaMessages::spellCheckerError
+        << Message::L_ERROR << msg;
 }
 
 template<typename T> void report_error(SernaMessages::Messages m, const T& arg)
 {
-    SpellErrorInfoPtr ip(new SpellChecker::Error::Info);
-    *ip << m << Message::L_ERROR << arg;
-    throw AspellErr(ip.get());
+    msgbox_stream() << m << Message::L_ERROR << arg;
 }
 
 AspellSpeller* AspellInstance::makeSpeller(const nstring& dict)
@@ -567,25 +477,28 @@ AspellSpeller* AspellInstance::makeSpeller(const nstring& dict)
     DDBG << "makeSpeller: dict=" << sqt(dict) << std::endl;
     initConfig();
 
-    if (dict_.empty() || dict_map_.empty())
+    if (dict_.empty() || dict_map_.empty()) {
         report_error(SernaMessages::spellCheckerNoDictionaries);
-
+        return 0;
+    }
     const nstring& cur_dict = dict.empty() ? dict_ : dict;
 
     DDBG << "makeSpeller: cur_dict=" << sqt(cur_dict) << std::endl;
 
-    if (0 == dict_map_.count(cur_dict))
+    if (0 == dict_map_.count(cur_dict)) {
         report_error(SernaMessages::spellCheckerInvalidDict,
                      from_latin1(cur_dict.data(), cur_dict.size()));
-
-    if (getEncoding(cur_dict).empty())
+        return 0;
+    }
+    if (getEncoding(cur_dict).empty()) {
         report_error(SernaMessages::spellCheckerEncodingError,
                      from_latin1(cur_dict.data(), cur_dict.size()));
-
+        return 0;
+    }
     AspellConfig* acp = FUN(aspell_config_clone)(getConfig());
     FUN(aspell_config_replace)(acp, "lang", cur_dict.c_str());
 
-    nstring base(local_8bit(get_cfg_value(cfg::SPELL_PWSDIR_VAR)));
+    nstring base(local_8bit(get_cfg_value(SPELL_PWSDIR_VAR)));
     base.append(1, PathName::DIR_SEP).append("serna.").append(cur_dict, 0, 2);
 
     nstring tmp(base + ".pws");
@@ -602,6 +515,7 @@ AspellSpeller* AspellInstance::makeSpeller(const nstring& dict)
         DDBG << "Aspell error message = " << FUN(aspell_error_message)(pche)
              << std::endl;
         report_error(pche);
+        return 0;
     }
     else {
         pche_g.dismiss();
@@ -626,13 +540,4 @@ const nstring& AspellInstance::getEncoding(const nstring& dict)
         fill_encoding(dict_map_, it, data_dir_);
     return encoding;
 }
-//!
-void* AspellResolver::resolve(const char* name)
-{
-    AspellLibrary& al = AspellLibrary::instance();
-    if (void* p = al.resolveSym(name))
-        return p;
-    throw AspellErr(al.getSymError());
-    return 0; // prevent possible compiler warning
-}
 
